import gnu.io.*;

import java.util.*;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame; 
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.Timer;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;

class Vrchol
{
	int x;
	int y;
	Vrchol lavyVrchol, pravyVrchol, dolnyVrchol, hornyVrchol;
	boolean lavySused, pravySused, dolnySused, hornySused;
	
	public Vrchol(int x, int y)
	{
		this.x = x;
		this.y = y;
	}
	
	public void nastavSusedov(boolean[] sus)
	{
		lavySused = sus[0];
		pravySused = sus[1];
		dolnySused = sus[2];
		hornySused = sus[3];
	}
	
	public String toString() 
	{
		return "(" + Integer.toString(x) + "," + Integer.toString(y) + ")";
	}
}


@SuppressWarnings("serial")
public class RobotFrame extends JFrame
{
	
	private int sizeX = 650;
	private int sizeY = 550;
	
	private JPanel infoPanel = new JPanel();
	private JPanel canvasPanel = new JPanel();
	private JPanel controlPanel = new JPanel();
	private static Canvas platnoMapa = new Canvas();
	private static Canvas platnoRealita = new Canvas();
	private BufferedImage buffImage = new BufferedImage(sizeX, sizeY + 2, BufferedImage.TYPE_INT_ARGB);
	
	private JButton butStart = new JButton("Start");
	private JButton butStop = new JButton("Stop");
	private JLabel lblStav = new JLabel("Najskr si polote robota do bludiska!");
	private JMenuBar jmb = new JMenuBar(); 
	private JMenu mMenu = new JMenu("Menu");
	private JMenuItem miAbout = new JMenuItem("About");
	private JMenuItem miExit = new JMenuItem("Exit");
	private JCheckBoxMenuItem miCanvasShow = new JCheckBoxMenuItem("Uk cestu robota");

	private FoxiMaxi foximaxi;
	private Timer pohybTimer = new Timer(5, new robotPohybTimer());
	private Timer kreslovac = new Timer(100, new kreslovacTimer()); 
	
	private Connection connection;
	private String mojPort;
	
	private Vrchol predchadzajuciVrchol, prvyVrchol;
	private HashMap<Vrchol, HashSet<Vrchol>> mapaBludiska = new HashMap<>();
	
	private LinkedList<Vrchol> najkratsiaCesta = new LinkedList<>();
	private LinkedList<Byte> najkratsiaCestaPrikazy = new LinkedList<>();
	
	class kreslovacTimer implements ActionListener
	{

		public void actionPerformed(ActionEvent e) 
		{
			platnoRealita.getGraphics().drawImage(foximaxi.mojaKresba, 0, 0, RobotFrame.this);
			platnoRealita.getGraphics().drawRect(0, 0, sizeX - 1, sizeY + 1);

			Graphics g = buffImage.getGraphics();
			g.setColor(Color.white);
			g.fillRect(0, 0, sizeX - 1, sizeY + 1);
			g.setColor(Color.black);
			g.drawRect(0, 0, sizeX - 1, sizeY + 1);
			g.drawString("Pocet vrcholov: " + mapaBludiska.size(), 20, 20);
			
			for (Iterator<Vrchol> iter = mapaBludiska.keySet().iterator(); iter.hasNext(); )
			{
				Vrchol vrchol = iter.next();
				g.setColor(Color.red);
				if (!vrchol.hornySused)  g.drawLine(vrchol.x, vrchol.y, vrchol.x, vrchol.y - 15);
				if (!vrchol.dolnySused)  g.drawLine(vrchol.x, vrchol.y, vrchol.x, vrchol.y + 15);
				if (!vrchol.lavySused)   g.drawLine(vrchol.x, vrchol.y, vrchol.x - 15, vrchol.y);
				if (!vrchol.pravySused)  g.drawLine(vrchol.x, vrchol.y, vrchol.x + 15, vrchol.y);
				g.setColor(Color.black);
				for (Iterator<Vrchol> iterSused = mapaBludiska.get(vrchol).iterator(); iterSused.hasNext(); )
				{
					Vrchol sused = iterSused.next();
					g.setColor(Color.black);
					g.drawLine(vrchol.x, vrchol.y, sused.x, sused.y);
					g.setColor((sused.equals(predchadzajuciVrchol)) ? Color.red : Color.black);
					g.fillOval(sused.x - 5, sused.y - 5, 10, 10);
				}
				g.setColor((vrchol.equals(predchadzajuciVrchol)) ? Color.red : Color.black);
				g.fillOval(vrchol.x - 5, vrchol.y - 5, 10, 10);
			}
			g.setColor(Color.blue);
			for (int i = 0; i < najkratsiaCesta.size(); ++i)
			{
				g.fillOval(najkratsiaCesta.get(i).x - 8, najkratsiaCesta.get(i).y - 8, 16, 16);
				if (i != najkratsiaCesta.size() - 1)
					g.drawLine(najkratsiaCesta.get(i).x, najkratsiaCesta.get(i).y, 
							najkratsiaCesta.get(i+1).x, najkratsiaCesta.get(i+1).y);
			}
			platnoMapa.getGraphics().drawImage(buffImage, 0, 0, RobotFrame.this);
		}
		
	}
	
	private void pridajHranu(Vrchol v1, Vrchol v2)
	{
		mapaBludiska.get(v1).add(v2);
		mapaBludiska.get(v2).add(v1);
	}
	
	private Vrchol pridajVrchol(int x, int y, boolean[] sus)
	{
		Vrchol v = new Vrchol(x, y);
		v.nastavSusedov(sus);
		mapaBludiska.put(v, new HashSet<Vrchol>());
		if (predchadzajuciVrchol != null)
			pridajHranu(v, predchadzajuciVrchol);
		nastavPredchadzajuci(v);
		predchadzajuciVrchol = v;
		return v;
	}
	
	private void rekonstruujNajkratsiuCestu(HashMap<Vrchol, Vrchol> cesta, Vrchol ciel)
	{
		najkratsiaCesta.addFirst(ciel);
		if (cesta.containsKey(ciel))
		{
			Vrchol dalsi = cesta.get(ciel);
			rekonstruujNajkratsiuCestu(cesta, dalsi);
		}
	}
	
	@SuppressWarnings("unchecked")
	private void rekonstruujNajkratsiuCestuPrikazy()
	{
		int kam = 0, smer = (foximaxi.direction == 0.0) ? 360 : (int) foximaxi.direction;;
		LinkedList<Vrchol> cesta = (LinkedList<Vrchol>) najkratsiaCesta.clone();
		Vrchol pred = cesta.removeFirst();
		while (cesta.size() != 0)
		{
			if (pred.hornyVrchol != null && pred.hornyVrchol.equals(cesta.getFirst())) kam = 0;
			else if (pred.pravyVrchol != null && pred.pravyVrchol.equals(cesta.getFirst())) kam = 90;
			else if (pred.lavyVrchol != null && pred.lavyVrchol.equals(cesta.getFirst())) kam = 270;
			else if (pred.dolnyVrchol != null && pred.dolnyVrchol.equals(cesta.getFirst())) kam = 180;
			
			int otocenie = smer - kam; 
			while (otocenie < 0)
				otocenie += 360;
			System.out.println(smer + " " + kam + " " + otocenie);
			switch (otocenie)
			{
				case 0:
					najkratsiaCestaPrikazy.addLast((byte) 3);
					break;
				case 90:
					najkratsiaCestaPrikazy.addLast((byte) 4);
					break;
				case 180:
					najkratsiaCestaPrikazy.addLast((byte) 5);
					break;
				case 270:
					najkratsiaCestaPrikazy.addLast((byte) 6);
					break;
			}
			smer = kam;
			pred = cesta.removeFirst();
		}
	}
	
	private void A_Star(Vrchol start, Vrchol ciel)
	{
		HashSet<Vrchol> visited = new HashSet<>();
		Prioritny_Front<Vrchol> openFront = new Prioritny_Front<>();
		HashMap<Vrchol, Double> g = new HashMap<>();
		HashMap<Vrchol, Double> f = new HashMap<>();
		HashMap<Vrchol, Vrchol> cesta = new HashMap<>();
		
		najkratsiaCesta.clear();
		g.put(start, 0.0);
		f.put(start, g.get(start) + vzdialenost(start.x, start.y, ciel.x, ciel.y));
		openFront.enqueue(start, f.get(start));
		while (!openFront.isEmpty())
		{
			Vrchol vyvoleny = openFront.front();
			if (vyvoleny.equals(ciel))  
			{
				rekonstruujNajkratsiuCestu(cesta, ciel);
				System.out.println(najkratsiaCesta);
				rekonstruujNajkratsiuCestuPrikazy();
				System.out.println(najkratsiaCestaPrikazy);
				najkratsiaCesta.removeFirst();
				return;
			}
			openFront.dequeue();
			visited.add(vyvoleny);
			for (Iterator<Vrchol> iterSused = mapaBludiska.get(vyvoleny).iterator(); iterSused.hasNext(); )
			{
				Vrchol sused = iterSused.next();
				double tempG = g.get(vyvoleny) + vzdialenost(vyvoleny.x, vyvoleny.y, sused.x, sused.y);
				if (visited.contains(sused) && tempG >= g.get(sused))
					continue;
				if (!openFront.contains(sused) || tempG < g.get(sused))
				{
					cesta.put(sused, vyvoleny);
					g.put(sused, tempG);
					f.put(sused, g.get(sused) + vzdialenost(sused.x, sused.y, ciel.x, ciel.y));
					if (!openFront.contains(sused))
						openFront.enqueue(sused, f.get(sused));
				}
			}
		}
	}
	
	private void posliCestu() throws InterruptedException
	{
		System.out.println("cakam signal");
		while (!connection.smer.equals("OK")) { Thread.sleep(50); }
		connection.smer = "";
		System.out.println("idem poslat");
		while (najkratsiaCestaPrikazy.size() != 0)
		{
			connection.prikaz = najkratsiaCestaPrikazy.removeFirst();
			while (!connection.smer.equals("OK")) { Thread.sleep(50); }
			connection.smer = "";
		}
		connection.prikaz = 8;
	}
	
	private boolean dalsiKrok()
	{
		Vrchol ciel = null;
		double minVzdialenost = Double.MAX_VALUE;
		for (Iterator<Vrchol> iter = mapaBludiska.keySet().iterator(); iter.hasNext(); )
		{
			Vrchol v = iter.next();
			if (!v.hornySused || !v.lavySused || !v.dolnySused || !v.pravySused) 
			{
				double vzdialenost = vzdialenost(predchadzajuciVrchol, v); 
				 if (vzdialenost < minVzdialenost)
				 {
					 ciel = v;
					 minVzdialenost = vzdialenost;
				 }
						 
			}
		}
		if (ciel != null)
		{
			A_Star(predchadzajuciVrchol, ciel);
			if (najkratsiaCesta.size() > 1) 
			{
			  Vrchol v = najkratsiaCesta.get(najkratsiaCesta.size() - 2);  //predpsoledny
			  foximaxi.jump(v.x, v.y);
			}
			return true;
		}
		return false;
	}
	
	public double vzdialenost(int x1, int y1, int x2, int y2)
	{
		return Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
	}
	
	public int vzdialenost(Vrchol u, Vrchol v) 
	{
		if (u == v) 
			return 0;
		HashSet<Vrchol> visited = new HashSet<>();
		LinkedList<Vrchol> queue = new LinkedList<>();
		LinkedList<Integer> distCounter = new LinkedList<>();
		visited.add(u); queue.add(u); distCounter.add(1);
		int vzdialenost = 1;
		while (!queue.isEmpty()) 
		{
			Vrchol vyhodeny = queue.removeFirst();
			vzdialenost = distCounter.removeFirst();
			for (Vrchol i : mapaBludiska.get(vyhodeny)) 
			{
				if (i == v) return vzdialenost;
				if (!visited.contains(i)) 
				{
					visited.add(i);
					queue.add(i);
					distCounter.add(vzdialenost + 1);
				}
			}
		}
		return -1;
	}
	
	private Vrchol existujeVrchol(int x, int y)
	{
		Vrchol najblizsiDobry = null;
		double najblizsiVzdialenost = Double.MAX_VALUE;
		for (Iterator<Vrchol> iter = mapaBludiska.keySet().iterator(); iter.hasNext(); )
		{
			Vrchol v = iter.next();
			double vzdialenost = vzdialenost(v.x, v.y, x, y); 
			if (vzdialenost < 12.0 && vzdialenost < najblizsiVzdialenost)
			{
				najblizsiDobry = v;
				najblizsiVzdialenost = vzdialenost;
			}
		}
		return (najblizsiDobry == null) ? null : najblizsiDobry;
	}
	
	private void zistiPredchadzajuci(boolean[] sus, boolean pridanie)
	{
		if (najkratsiaCesta.size() != 0)
		{
			predchadzajuciVrchol = najkratsiaCesta.removeFirst();
			if (najkratsiaCesta.size() != 0)
				return;
		}
		if (!pridanie) return;
		Vrchol v = existujeVrchol((int) foximaxi.x, (int) foximaxi.y);
		if (v == null) 
			pridajVrchol((int) foximaxi.x, (int) foximaxi.y, sus);
		else
		{
			nastavPredchadzajuci(v);
			predchadzajuciVrchol = v;
			foximaxi.jump(v.x, v.y);
		}
	}
	
	private void nastavPredchadzajuci(Vrchol v)
	{
		if (predchadzajuciVrchol == null || v == predchadzajuciVrchol) 
			return;
		pridajHranu(v, predchadzajuciVrchol);
		switch ((int) foximaxi.direction)
		{
			case 0:
				v.dolnySused = true;
				v.dolnyVrchol = predchadzajuciVrchol;
				predchadzajuciVrchol.hornySused = true;
				predchadzajuciVrchol.hornyVrchol = v;
				break;
			case 90:
				v.lavySused = true;
				v.lavyVrchol = predchadzajuciVrchol;
				predchadzajuciVrchol.pravySused = true;
				predchadzajuciVrchol.pravyVrchol = v;
				break;
			case 180:
				v.hornySused = true;
				v.hornyVrchol = predchadzajuciVrchol;
				predchadzajuciVrchol.dolnySused = true;
				predchadzajuciVrchol.dolnyVrchol = v;
				break;
			case 270:
				v.pravySused = true;
				v.pravyVrchol = predchadzajuciVrchol;
				predchadzajuciVrchol.lavySused = true;
				predchadzajuciVrchol.lavyVrchol = v;
				break;
		}
	}
	
	public boolean mozemHore()
	{
		return (!predchadzajuciVrchol.hornySused && foximaxi.direction == 0.0) ||
			   (!predchadzajuciVrchol.dolnySused && foximaxi.direction == 180.0) ||
			   (!predchadzajuciVrchol.lavySused && foximaxi.direction == 270.0) ||
			   (!predchadzajuciVrchol.pravySused && foximaxi.direction == 90.0);
	}
	
	public boolean mozemVlavo()
	{
		return (!predchadzajuciVrchol.hornySused && foximaxi.direction == 90.0) ||
			   (!predchadzajuciVrchol.dolnySused && foximaxi.direction == 270.0) ||
			   (!predchadzajuciVrchol.lavySused && foximaxi.direction == 0.0) ||
			   (!predchadzajuciVrchol.pravySused && foximaxi.direction == 180.0);
	}
	
	public boolean mozemVpravo()
	{
		return (!predchadzajuciVrchol.hornySused && foximaxi.direction == 270.0) ||
			   (!predchadzajuciVrchol.dolnySused && foximaxi.direction == 90.0) ||
			   (!predchadzajuciVrchol.lavySused && foximaxi.direction == 180.0) ||
			   (!predchadzajuciVrchol.pravySused && foximaxi.direction == 0.0);
	}
	
	class robotPohybTimer implements ActionListener
	{

		public void actionPerformed(ActionEvent e)
		{
			if (connection.smer.length() == 0) return;
			switch(connection.smer.charAt(0)) 
			{
			case 'O' :
				if (!butStop.isEnabled())
					butStart.setEnabled(true);
				lblStav.setText("Robot mapuje!");
				break;
			case 'F' :
				foximaxi.go();
 				break;
			case 'L' :
				zistiPredchadzajuci(new boolean[] {true, true, true, true}, true);
				foximaxi.turnLeft();
				break;
			case 'R' :
				zistiPredchadzajuci(new boolean[] {true, true, true, true}, true);
				foximaxi.turnRight();
				break;
			case 'K' :
				zistiPredchadzajuci(new boolean[] {true, true, true, true}, true);
				if (dalsiKrok()) 
				{ 
					connection.prikaz = 7;
					try {
						posliCestu();
					} catch (InterruptedException e1) {
						e1.printStackTrace();
					}
				}
				else {
					connection.prikaz = 9;
					lblStav.setText("Hur! Robot namapoval bludisko, mete ho klikanm do mapy posla kam chcete");
				}
				break;	
			case 'Q' :
				foximaxi.atYourBack(true);
				break;
			case 'X' :
				zistiPredchadzajuci(new boolean[] {
						(foximaxi.direction == 90.0) ? true : false, 
						(foximaxi.direction == 270.0) ? true : false, 
						(foximaxi.direction == 0.0) ? true : false, 
						(foximaxi.direction == 180.0) ? true : false}, true);
				if (mozemHore())  connection.prikaz = 3;
				else if (mozemVlavo()) connection.prikaz = 4;
				else if (mozemVpravo()) connection.prikaz = 6;
				else if (dalsiKrok()) 
				{ 
					connection.prikaz = 7;
					try {
						posliCestu();
					} catch (InterruptedException E) { } 
				}
				else 
					connection.prikaz = 9;
				foximaxi.intersection(true, true, true);
				break;
			case 'T' :	
				zistiPredchadzajuci(new boolean[] {
						(foximaxi.direction == 90.0 || foximaxi.direction == 270.0) ? true : false, 
						(foximaxi.direction == 90.0 || foximaxi.direction == 270.0) ? true : false, 
						(foximaxi.direction == 0.0 || foximaxi.direction == 180.0) ? true : false, 
						(foximaxi.direction == 0.0 || foximaxi.direction == 180.0) ? true : false,}, true);
				if (mozemVlavo()) connection.prikaz = 4;
				else if (mozemVpravo()) connection.prikaz = 6;
				else if (dalsiKrok()) 
				{ 
					connection.prikaz = 7;
					try {
						posliCestu();
					} catch (InterruptedException E) { } 
				}
				else 
					connection.prikaz = 9;
				foximaxi.intersection(false, true, true);
				break;
			case 'A' :
				zistiPredchadzajuci(new boolean[] {
						(foximaxi.direction == 180.0 || foximaxi.direction == 90.0) ? true : false, 
						(foximaxi.direction == 270.0 || foximaxi.direction == 0.0) ? true : false, 
						(foximaxi.direction == 0.0 || foximaxi.direction == 90.0) ? true : false, 
						(foximaxi.direction == 270.0 || foximaxi.direction == 180.0) ? true : false,}, true);
				if (mozemHore()) connection.prikaz = 3;
				else if (mozemVlavo()) connection.prikaz = 4;
				else if (dalsiKrok()) 
				{ 
					connection.prikaz = 7;
					try {
						posliCestu();
					} catch (InterruptedException E) { } 
				}
				else
					connection.prikaz = 9;
				foximaxi.intersection(true, true, false);
				break;
			case 'B' :
				System.out.println("B");
				zistiPredchadzajuci(new boolean[] {
						(foximaxi.direction == 90.0 || foximaxi.direction == 0.0) ? true : false, 
						(foximaxi.direction == 180.0 || foximaxi.direction == 270.0) ? true : false, 
						(foximaxi.direction == 0.0 || foximaxi.direction == 270.0) ? true : false, 
						(foximaxi.direction == 180.0 || foximaxi.direction == 90.0) ? true : false,}, true);
				if (mozemHore()) connection.prikaz = 3;
				else if (mozemVpravo()) connection.prikaz = 6;
				else if (dalsiKrok()) 
				{ 
					connection.prikaz = 7;
					try {
						posliCestu();
					} catch (InterruptedException E) { } 
				}
				else {
					connection.prikaz = 9;
					mapaBludiska.remove(prvyVrchol);
				}
				foximaxi.intersection(true, false, true);
				break; 
			case 'C' :
				System.out.println("C");
				zistiPredchadzajuci(null, false);
				foximaxi.headLeft(90);
				break;
			case 'D' :
				System.out.println("D");
				zistiPredchadzajuci(null, false);
				foximaxi.headRight(90);
				break;
			case 'W' :
				System.out.println("W");
				zistiPredchadzajuci(null, false);
				break;
			case 'U' :
				System.out.println("U");
				Vrchol posl = najkratsiaCesta.getLast();
				predchadzajuciVrchol = posl;
				foximaxi.jump(posl.x, posl.y);
				najkratsiaCesta.clear();
				najkratsiaCestaPrikazy.clear();
				break;
			}
			if (connection.smer.length() > 1)
				switch (connection.smer.charAt(1)) 
				{  
				case 'C' :
					foximaxi.headLeft(90);
					break;
				case 'D' :
					foximaxi.headRight(90);
					break;
				}
			if (connection.smer.length() != 0 && connection.smer.charAt(0) != 'F')
				connection.smer = "";
		}
	
	}
	
	class splashScreen extends Window
	{
		
		Image image = (new ImageIcon("res/robot.jpg")).getImage();
		
		public splashScreen()
		{
			super(null); 
			setVisible(true);
			setUndecorated(true);
			setSize(image.getWidth(RobotFrame.this), image.getHeight(RobotFrame.this));
			setLocationRelativeTo(null);
			repaint();
		}
		
		public void paint(Graphics g)  
		{  
			g.drawImage(image, 0, 0, RobotFrame.this);  
		}  
		
	}
	
	public static void infoBox(String info, String hlavicka)
    {
        JOptionPane.showMessageDialog(null, info, hlavicka, JOptionPane.INFORMATION_MESSAGE);
    }
	
	public void vyberPort(String[] ports)
	{
		mojPort = (String) JOptionPane.showInputDialog(
                new JFrame(),
                "Vyberte si sriov port, o budete pouva na komunikciu!"
                + "\nDostupn porty:",
                "Zadajte port",
                JOptionPane.PLAIN_MESSAGE,
                UIManager.getIcon("OptionPane.informationIcon"),
                ports, ports[0]);
	}
    
    @SuppressWarnings("unchecked")
	static String[] listPorts()
    {
    	Vector<String> porty = new Vector<>();
        java.util.Enumeration<CommPortIdentifier> portEnum = CommPortIdentifier.getPortIdentifiers();
        while ( portEnum.hasMoreElements() ) 
        {
            CommPortIdentifier portIdentifier = portEnum.nextElement();
            porty.add(portIdentifier.getName());
            System.out.println(portIdentifier.getName());
        }        
        return porty.toArray(new String[0]);
    }
    
	public RobotFrame()
	{
		
		splashScreen s = new splashScreen();
		String[] ports = listPorts();
		while (ports.length == 0)
		{
			infoBox("Nenaiel som iaden sriov port ... nezabudli ste zapn bluetooth?", 
					"Kde s porty?");
			ports = listPorts();
		}
		s.dispose();
		setUndecorated(false);
		vyberPort(ports);
		
		while (true)
		{
			try
	        {
				connection = new Connection();
				connection.connect(mojPort);
				break;
	        }
	        catch (Exception e)
	        {
	            e.printStackTrace();
	            infoBox("Bluetoothov spojenie zlyhalo ... ", 
	            		"Nenaiel som robota");
	            vyberPort(ports);
	        }
		}
		
		//connection = new Connection();
		//addKeyListener(new klavesnica());
		
		setSize(sizeX + 25, sizeY + 185);
	    setVisible(true);
	    setResizable(false);
	    setAlwaysOnTop(true);
	    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	    setLayout(new BorderLayout());
		setTitle("AIROB projekt");
		
		jmb.add(mMenu);
		mMenu.add(miCanvasShow);
		mMenu.addSeparator();
		mMenu.add(miAbout);
		mMenu.add(miExit);
		setJMenuBar(jmb);
		
		platnoMapa.setSize(sizeX, sizeY);
		platnoMapa.setVisible(true);
		platnoMapa.setBackground(Color.white);
		platnoMapa.addMouseListener(new mouseKlik());
		
		platnoRealita.setSize(sizeX, sizeY);
		platnoRealita.setVisible(false);
		platnoRealita.setBackground(Color.white);
		
		canvasPanel.setLayout(new BorderLayout());
		canvasPanel.setBorder(new EmptyBorder(10, 10, 10, 10));
		canvasPanel.add(platnoMapa, "West");
		canvasPanel.add(platnoRealita, "East");
		add(canvasPanel, "Center");
		
		
		infoPanel.setLayout(new FlowLayout());
		infoPanel.setBorder(new EmptyBorder(10, 10, 10, 10));
		infoPanel.add(lblStav);
		
		controlPanel.setLayout(new FlowLayout());
		((FlowLayout)controlPanel.getLayout()).setHgap(25);
		controlPanel.setBorder(new EmptyBorder(15, 10, 15, 10));
		butStart.setEnabled(false);
		controlPanel.add(butStart);
		butStop.setEnabled(false);
		controlPanel.add(butStop);
		
		add(infoPanel, "North");
		add(controlPanel, "South");
		
		kreslovac.start();
		pohybTimer.start();
        foximaxi = new FoxiMaxi(platnoRealita);
        prvyVrchol = new Vrchol(sizeX / 2, sizeY / 2);
        prvyVrchol.nastavSusedov(new boolean[] {true, true, true, true});
        mapaBludiska.put(prvyVrchol, new HashSet<Vrchol>());
        predchadzajuciVrchol = prvyVrchol;
		
		butStart.addActionListener(new ActionListener() 
		{
			public void actionPerformed(ActionEvent e) 
			{
				connection.prikaz = 1;
				pohybTimer.start();
				butStart.setEnabled(false);
				butStop.setEnabled(true);
			}
		});
		
		butStop.addActionListener(new ActionListener() 
		{
			public void actionPerformed(ActionEvent e)
			{
				pohybTimer.stop();
				connection.prikaz = 2;
				butStart.setEnabled(true);
				butStop.setEnabled(false);
		    }
		});
		
		miCanvasShow.addMouseListener(new MouseAdapter() 
		{
			public void mouseReleased(MouseEvent e) 
			{
				platnoRealita.setVisible(miCanvasShow.isSelected());
				RobotFrame.this.setSize(((miCanvasShow.isSelected()) ? 2 : 1) * sizeX + 25, sizeY + 185);
		    }
		});
		
        miExit.addActionListener(new ActionListener()
        {
        	public void actionPerformed(ActionEvent e)
        	{
        		System.exit(0);
        	}
        });
        
	}
	
	class mouseKlik extends MouseAdapter
	{	
		public void mouseReleased(MouseEvent e) 
		{
			for (Iterator<Vrchol> iter = mapaBludiska.keySet().iterator(); iter.hasNext(); )
			{
				Vrchol v = iter.next();
				if (vzdialenost(e.getX(), e.getY(), v.x, v.y) < 15)
				{
					A_Star(predchadzajuciVrchol, v);
					connection.prikaz = 7;
					try {
						posliCestu();
					} catch (InterruptedException E) { } 
					break;
				}
			}
		}
	}
	
	class klavesnica extends KeyAdapter
	{
		public void keyReleased(KeyEvent e) 
		{
			switch(e.getKeyChar())
			{
				case 'a':
					connection.smer = "L";
					break;
				case 'd':
					connection.smer = "R";
					break;
				case 'w':
					connection.smer = "F";
					break;
				case 's':
					connection.smer = "K";
					break;
				case 'x':
					connection.smer = "X";
					break;
				case 't':
					connection.smer = "T";
					break;
				case 'n':
					connection.smer = "A";
					break;
				case 'm':
					connection.smer = "B";
					break;
				case 'o' :
					foximaxi.headLeft(90);
					break;
				case 'p' :
					foximaxi.headRight(90);
					break;
				case 'q':
					connection.smer = "Q";
					break;
			}
		}
	}
	
	public static void main(String[] args)
	{
		new RobotFrame();
	}
	
}
